# Atom->JSON Feed converter
# Expects xmltodict JSON output as input, outputs a JSON Feed 1.1 feed backwards-compatible with 1.0
# Does not support entries without links.
# Required variables:
# $feed_url: Absolute URL to the resulting JSON Feed

def ensure_string: if type == "array" then .[0] else . end | if type == "object" then .["#text"] else . end;
def ensure_array: (. // []) | if type == "array" then . else [.] end;
def find_links(rel): [(.link // []) | if type == "array" then .[] else . end | select(.["@rel"] == rel)];
def parse_authors: [
        ((.author | ensure_array) + (.contributor | ensure_array))[]
        | . as $author
        | {"name": .name}
        | if $author.uri then .url = $author.uri else . end
    ];

.feed | . as $feed | {
    "version": "https://jsonfeed.org/version/1.1",
    "title": (.title | ensure_string),
    "home_page_url": find_links("alternate")[0]["@href"],
    "feed_url": $feed_url,
    "description": (.description | ensure_string // ""),
    "expired": false,
    "authors": parse_authors,
    "items": [
        .entry | ensure_array[] | select(.link) | . as $entry | {
            "id": .id,
            "title": (.title | ensure_string),
            "url": find_links("alternate")[0]["@href"],
            "date_published": (.published // .updated),
            "date_modified": .updated,
            "authors": parse_authors,
            "tags": [
                .category | ensure_array[] | .["@term"]
            ],
            "attachments": [
                find_links("enclosure")[] | {
                    "url": .["@href"],
                    "size_in_bytes": .["@length"],
                    "mime_type": .["@type"]
                }
            ],
            "content_html": (.content | ensure_array[0] | ensure_string // "")
        }
        # Optional summary
        | if $entry.summary then .summary = ($entry.summary | ensure_string // "") else . end
        # Optional external URL
        | if ($entry|find_links("related")[0]) then .external_url = ($entry|find_links("related")[0]["@href"]) else . end
    ]
}
# Optional icon
| if $feed.icon then .icon = $feed.icon else . end
