# Atom 1.0 to RSS 2.0 converter
# Expects xmltodict JSON output, returns xmltodict-compatible JSON

def ensure_string: if type == "array" then .[0] else . end | if type == "object" then .["#text"] else . end;
def ensure_array: (. // []) | if type == "array" then . else [.] end;
def find_links(rel): [(.link // []) | if type == "array" then .[] else . end | select(.["@rel"] == rel)];
def parse_categories: [
    .category
    | ensure_array[]
    | { "#text": .["@term"] }
    + (if .["@scheme"] then {"@domain": .["@scheme"]} else {} end)
];
def parse_date: (try (.updated // .published) | fromdateiso8601 | strftime("%a, %d %b %Y %T %z")) // [];

{
    "rss": {
        "@version": "2.0",
        "channel": (
            .feed as $feed
            | .feed
            | {
                "title": .title,
                "link": (find_links("self")[0]["@href"] // .id),
                "pubDate": parse_date,
                "lastBuildDate": parse_date,
                "category": parse_categories,
                "generator": (.generator | ensure_string // []),
                "description": (.subtitle | ensure_string // []),
                "copyright": (.rights | ensure_string // []),
                "language": (.["@xml:lang"] | ensure_string // []),
                "item": [
                    .entry[]
                    | {
                        "title": .title,
                        "guid": {
                            "@isPermaLink": "false",
                            "#text": .id
                        },
                        "link": (find_links("alternate")[0]["@href"]? // []),
                        "pubDate": parse_date,
                        "description": ((.summary // .content) | ensure_string | []),
                        "author": ((
                            # Pick the first author tag, falling back to the feed's authors if needed
                            ((.author | ensure_array) + ($feed.author | ensure_array))[0] // {}
                            | if .email then "\(.email) (\(.name))" else .name end
                        ) // []),
                        "category": parse_categories,
                        "source":  (if .source then {
                            "@url": .id,
                            "#text": .title
                        } else [] end),
                        "enclosure": [
                            find_links("enclosure")[]
                            | {
                                "@url": .["@href"],
                                "@type": (.["@type"] | ensure_string // []),
                                "@length": (.["@length"] | ensure_string // [])
                            }
                        ]
                    }
                ]
            }
            | . + if ($feed.logo // $feed.icon) then {
                "image": {
                    "title": .title,
                    "link": .link,
                    "url": ($feed.logo // $feed.icon)
                }
            } else {} end
        )
    }
}
