# RSS->CDF converter
# Expects xmltodict JSON output as input, outputs a CDF feed
# Supports MediaRSS.

def ensure_string: if type == "array" then .[0] else . end | if type == "object" then .["#text"] else . end;
def ensure_array: (. // []) | if type == "array" then . else [.] end;
def parse_date: ensure_string | strptime("%a, %d %b %Y %T %Z") | mktime | todateiso8601[:16];

{"CHANNEL": (
    .rss.channel | . as $channel | {
        "A": {
            "@HREF": (.link | ensure_string)
        },
        "ABSTRACT": (.description | ensure_string),
        "TITLE": (.title | ensure_string),
        "ITEM": [
            .item | ensure_array[] | . as $item | {
                "TITLE": .title,
                "USAGE": {"@VALUE": "Channel"},
                "@SHOW": "Channel"
            }
            | if $item.description then .ABSTRACT = ($item.description | ensure_string) else . end
            | if $item.link then
                .A = {"@HREF": ($item.link | ensure_string)}
            elif $item.enclosure then
                .A = {"@HREF": $item.enclosure["@url"]}
            else . end
            | if $item.pubDate then .["@LASTMOD"] = ($item.pubDate | parse_date) else . end
            | if $item["media:thumbnail"] then .LOGO = {
                "@HREF": $item["media:thumbnail"]["@url"],
                "@STYLE": "IMAGE"
            } else . end
        ]
    }
    | if $channel.lastBuildDate // $channel.pubDate then .["@LASTMOD"] = (
        $channel.lastBuildDate // $channel.pubDate | parse_date
    ) else . end
    | if $channel.image then .LOGO = {
        "@HREF": $channel.image.url,
        "@STYLE": "IMAGE"
    } else . end
)}
