# RSS->Echo converter
# Expects xmltodict JSON output as input, outputs an Echo feed

def ensure_string: if type == "array" then .[0] else . end | if type == "object" then .["#text"] else . end;
def ensure_array: (. // []) | if type == "array" then . else [.] end;
def parse_date: ensure_string | strptime("%a, %d %b %Y %T %Z") | mktime | todateiso8601;

{"feed": (
    .rss.channel | . as $channel | {
        "@xmlns": "http://purl.org/echo/",
        "@version": "1.0",
        "title": .title,
        "summary": .description,
        "link": [
            {
                "@rel": "alternate",
                "@type": "text/html",
                "@href": (.link | ensure_string),
            }
        ],
        "entry": [
            .item | ensure_array[] | . as $item | {
                "@id": (.guid // .link // .title | ensure_string),
                "title": (.title | ensure_string),
                "link": []
            }
            | if $item.author then .author = {"name": ($item.author | ensure_string)} else . end
            | if $item.pubDate then
                .issued = ($item.pubDate | parse_date)
                | .created = .issued
                | .modified = .issued
            else . end
            | if $item.link then .link += [{
                "@rel": "alternate",
                "@type": "text/html",
                "@href": ($item.link | ensure_string)
            }] else . end
            | if $item.source then .link += [{
                "@rel": "source",
                "@type": "text/html",
                "@href": ($item.source | ensure_string)
            }] else . end
            | if $item.description then
                .content = {
                    "@type": "text/html",
                    "#text": ($item.description | ensure_string)
                }
            else . end
            | if $item.enclosure then
                . as $entry | .content = (
                    {
                        "@type": $item.enclosure["@type"],
                        "@href": $item.enclosure["@url"]
                    }
                    # Wrap in a multipart if the entry already has a content
                    | if $entry.content then {
                        "@type": "multipart/related",
                        "content": [$entry.content, .]
                    } else . end
                )
            else . end
        ]
    }
    | if $channel.language then .["@xml:lang"] = ($channel.language | ensure_string) else . end
    | if $channel.image then .link += [{
        "@rel": "logo",
        # RSS does not provide an image's mime type, so we try to guess
        "@type": ("image/" + ($channel.image.url | capture("\\.(?<ext>[a-zA-Z0-9]+)$").ext // "x-unknown")),
        "@href": $channel.image.url
    }] else . end
)}
