# RSS->JSON Feed converter
# Expects xmltodict JSON output as input, outputs a JSON Feed 1.1 feed backwards-compatible with 1.0
# Does not support RSS items without links.
# Required variables:
# $feed_url: Absolute URL to the resulting JSON Feed

def ensure_string: if type == "array" then .[0] else . end | if type == "object" then .["#text"] else . end;
def ensure_array: (. // []) | if type == "array" then . else [.] end;

.rss.channel | . as $channel | {
    "version": "https://jsonfeed.org/version/1.1",
    "title": (.title | ensure_string),
    "home_page_url": (.link | ensure_string),
    "feed_url": $feed_url,
    "description": (.description | ensure_string),
    "expired": false,
    "items": [
        .item | ensure_array[] | select(.link) | . as $item | {
            "id": (.guid // .link | ensure_string),
            "title": .title,
            "url": .link,
            "content_html": (.description // "" | ensure_string),
            "tags": [
                .category | ensure_array[] | ensure_string
            ],
            "attachments": [
                .enclosure | ensure_array[] | {
                    "url": .["@url"],
                    "size_in_bytes": .["@length"],
                    "mime_type": .["@type"]
                }
            ]
        }
        # Optional publication date
        | if $item.pubDate then .date_published = ($item.pubDate | strptime("%a, %d %b %Y %T %Z") | mktime | todateiso8601) else . end
        # Optional author
        | if $item.author then .author = {"name": $item.author} | .authors = [.author] else . end
    ]
}
# Optional language
| if $channel.language then .language = $channel.language else . end
# Optional icon
| if $channel.image then .icon = $channel.image.url else . end
